@@ -4,7 +4,7 @@ from .dbobjects2 import (
 
 from .common import (
     AlreadyExistsError, NotFoundError, IntegrityError, MetaValidationError, 
-    parse_name, limited, alias, make_list_if_short, insert_bulk
+    parse_name, alias, make_list_if_short, insert_bulk
 )
 
 from .param_category import DBParamCategory
\ No newline at end of file
