@@ -1,5 +1,5 @@
 import uuid, json, hashlib, re, time, io, traceback, base64
-from metacat.util import to_bytes, to_str, epoch, chunked
+from metacat.util import to_bytes, to_str, epoch, chunked, limited, strided, skipped, first_not_empty
 from metacat.auth import BaseDBUser
 from psycopg2 import IntegrityError
 
@@ -13,9 +13,11 @@ from .common import (
     DBObject, _DBManyToMany,
     AlreadyExistsError, DatasetCircularDependencyDetected, NotFoundError, MetaValidationError,
     parse_name, fetch_generator, alias, 
-    first_not_empty, limited, strided, skipped, insert_bulk
+    insert_bulk
 )
 
+
+
 class DBFileSet(object):
     
     def __init__(self, db, files=[]):
@@ -174,14 +176,14 @@ class DBFileSet(object):
         elif basic_file_query.Limit is not None:
             limit = min(limit, basic_file_query.Limit)
             
-        dataset_selector = basic_file_query.DatasetSelector
+        bdq = basic_file_query.DatasetSelector
         datasets = None
-        if dataset_selector is not None:
-            datasets = list(basic_file_query.DatasetSelector.datasets(db))
+        if bdq is not None:
+            datasets = list(bdq.datasets(db))
             if not datasets:
                 return DBFileSet(db)      # empty File Set
 
-        if dataset_selector is None:
+        if bdq is None:
             return DBFileSet.all_files(db, dnf, with_metadata, limit)
             
         elif len(datasets) == 1:
@@ -215,7 +217,10 @@ class DBFileSet(object):
 
         file_meta_exp = MetaExpressionDNF(basic_file_query.Wheres).sql(f) or "true"
 
-        datasets = None if basic_file_query.DatasetSelectors is None else DBDataset.datasets_for_selectors(db, basic_file_query.DatasetSelectors)
+        datasets = None if basic_file_query.DatasetSelectors is None else DBDataset.datasets_for_bdqs(db, basic_file_query.DatasetSelectors)
+        datasets = list(datasets)
+        debug("sql_for_basic_query: datasets:", datasets)
+        
         attrs = DBFile.attr_columns(f)
         if datasets is None:
             # no dataset selection
@@ -350,8 +355,14 @@ class DBFile(object):
     
     @staticmethod
     def generate_id():
-        return uuid.uuid4().hex
-        
+        #return uuid.uuid4().hex
+        done = False
+        x = None
+        while not done:
+            x = base64.b64encode(uuid.uuid4().bytes, b"__")[:16]            # 62**16 = 100 lifetimes of the Universe in nanoseconds
+            done = b'_' not in x
+        return x.decode("utf-8")
+
     def __str__(self):
         return "[DBFile %s %s:%s]" % (self.FID, self.Namespace, self.Name)
         
@@ -1406,15 +1417,50 @@ class DBDataset(DBObject):
         return DBDataset.get_many(db, [spec.split(":",1) for spec in specs])    
 
     @staticmethod    
-    def apply_dataset_selector(db, dataset_selector, limit):
+    def _______expand_dataset_selector(db, dataset_selector, limit):
         patterns = dataset_selector.Patterns
         with_children = dataset_selector.WithChildren
         recursively = dataset_selector.Recursively
         datasets = DBDataset.list_datasets(db, patterns, with_children, recursively)
         return limited(dataset_selector.filter_by_having(datasets), limit)
-        
+
     @staticmethod
-    def dataset_selector_sql(namespace, name, pattern, with_children, recursively, meta_filter):
+    def datasets_for_bdq(db, bdq, limit=None):
+        #print("datasets_for_bdq: bdq:", bdq)
+        if not (bdq.Namespace and bdq.Name):
+            name_or_pattern = bdq.Name
+            raise ValueError(f"Dataset specification error: {selector.Namespace}:{name_or_pattern}")
+        if bdq.is_explicit():
+            debug("datasets_for_bdq: bdq is explicit")
+            ds = DBDataset.get(db, bdq.Namespace, bdq.Name)
+            if ds is None:
+                out = []
+            else:
+                out = [ds]
+        else:
+            sql = DBDataset.sql_for_basic_dataset_query(bdq)
+            debug("datasets_for_bdq: sql: " + sql)
+            c = db.cursor()
+            c.execute(sql)
+            out = limited((DBDataset.from_tuple(db, tup) for tup in fetch_generator(c)), limit)
+        return out
+
+    @staticmethod
+    def datasets_for_bdqs(db, bdq_list):
+        for bdq in bdq_list:
+            debug("datasets_for_bdqs: bdq:", bdq)
+            for ds in DBDataset.datasets_for_bdq(db, bdq):
+                yield ds
+
+    @staticmethod
+    def sql_for_basic_dataset_query(bdq):
+            namespace = bdq.Namespace
+            name = bdq.Name
+            pattern = bdq.Pattern
+            regexp = bdq.RegExp
+            with_children = bdq.WithChildren
+            recursively = bdq.Recursively
+            where = bdq.Where
 
             pc = alias("pc")
             pc1 = alias("pc1")
@@ -1425,15 +1471,21 @@ class DBDataset(DBObject):
 
             columns = DBDataset.columns(as_text=False)
 
-            name_cmp = f"{ds}.name = '{name}'" if not pattern else f"{ds}.name like '{name}'"
-        
+            meta_filter_dnf = MetaExpressionDNF(where) if where is not None else None
+
+            name_cmp_op = "=" if not pattern else (
+                "~" if regexp else "like"
+            )
+
+            name_cmp = f"{ds}.name {name_cmp_op} '{name}'"
+
             if not with_children:
                 #print([f"{ds}.{c}" for c in columns])
                 columns = ",".join([f"{ds}.{c}" for c in columns])
                 #print("columns:", columns)
                 sql = f"select {columns} from datasets {ds} where {ds}.namespace = '{namespace}' and {name_cmp}"
-                if meta_filter is not None:
-                    sql += " and " + meta_filter.sql(ds)
+                if meta_filter_dnf is not None:
+                    sql += " and " + meta_filter_dnf.sql(ds)
             else:
                 columns = ",".join(f"{d}.{c}" for c in columns)
                 top_sql = f"""select {ds}.namespace, {ds}.name, array[{ds}.namespace || ':' || {ds}.name], false
@@ -1470,7 +1522,7 @@ class DBDataset(DBObject):
                                         where {pc1}.parent_namespace = {s}.namespace and {pc1}.parent_name = {s}.name and not {s}.loop
                                 ) -- 1
                     """
-                meta_condition = "and " + meta_filter.sql(d) if meta_filter is not None else ""
+                meta_condition = "and " + meta_filter_dnf.sql(d) if meta_filter_dnf is not None else ""
                 sql = f"""
                     {selected_sql}
                     select distinct {columns} 
@@ -1479,36 +1531,9 @@ class DBDataset(DBObject):
                             {d}.name = {s}.name
                             {meta_condition}
                 """
-            debug("sql_for_selector(", namespace, name, pattern, with_children, recursively, meta_filter, ")\n", sql)
+            debug(f"sql_for_basic_dataset_query({bdq}): sql:\n", sql)
             return sql
 
-    @staticmethod
-    def datasets_for_selector(db, selector):
-        #print("datasets_for_selector: selector:", selector)
-        if not (selector.Namespace and (selector.Name or selector.Pattern)):
-            name_or_pattern = selector.Name or selector.Pattern
-            raise ValueError(f"Dataset specification error: {selector.Namespace}:{name_or_pattern}")
-        if selector.is_explicit():
-            ds = DBDataset.get(db, selector.Namespace, selector.Name)
-            if ds is None:
-                out = []
-            else:
-                out = [ds]
-        else:
-            sql = DBDataset.dataset_selector_sql(selector.Namespace, selector.Name, selector.Pattern, selector.WithChildren,
-                    selector.Recursively, selector.Having)
-            c = db.cursor()
-            c.execute(sql)
-            out = [DBDataset.from_tuple(db, tup) for tup in fetch_generator(c)]
-        return out
-        
-    @staticmethod
-    def datasets_for_selectors(db, selectors):
-        out = {}        # (namespace, name) -> Dataset, to filter out duplicates
-        for sel in selectors:
-            for ds in DBDataset.datasets_for_selector(db, sel):
-                out[(ds.Namespace, ds.Name)] = ds
-        return list(out.values())
         
     def validate_file_metadata(self, meta):
         """
