@@ -18,37 +18,59 @@ def parse_name(name, default_namespace=None):
 
 undid = parse_name
 
-class ServerError(Exception):
+class MCError(Exception):
+    pass
+
+class WebAPIError(MCError):
     
-    def __init__(self, url, status_code, message, body=""):
+    Headline = "MetaCat API error"
+    
+    def __init__(self, url, response):
         self.URL = url
-        self.StatusCode = status_code
-        self.Message = message
-        self.Body = to_str(body)
+        self.StatusCode = response.status_code
+        self.Message = None
+        self.Body = to_str(response.text)
+        if response.headers.get("content-type") in ("text/json", "application/json"):
+            try:    self.Data = json.loads(response.text)
+            except: self.Data = None
+            self.Message = self.Data.get("message", "")
+        else:
+            self.Message = to_str(response.text)
         
     def __str__(self):
-        #msg = f"MetaCatServer error:\n  URL: {self.URL}\n  HTTP status code: {self.StatusCode}\n  {self.Message}"
-        msg = self.Message
-        if self.Body:
-            msg += "\nMessage from the server:\n"+self.Body+"\n"
-        return msg
-        
-class WebAPIError(ServerError):
-    
-    def __init__(self, url, status_code, body):
-        ServerError.__init__(self, url, status_code, "", body)
-    
+        lines = []
+        if self.Message:
+            lines.append(self.Message)
+        else:
+            lines.append(self.Body)
+        return "\n".join(lines)
+
     def json(self):
-        #print("WebAPIError.json: body:", self.Body)
-        return json.loads(self.Body)
+        return self.Data
+        
+class ServerReportedError(WebAPIError):
+    Headline = "Server side application error"
+
+    def __init__(self, url, status_code, type, value):
+        message = type
+        if value:
+            message += ": " + value
+        WebAPIError.__init__(self, url, status_code, message=message)
+
+class InvalidArgument(WebAPIError):
+    Headline = "Invalid argument"
         
 class NotFoundError(WebAPIError):
-    pass
+    Headline = "Object not found"
+
+class BadRequestError(WebAPIError):
+    Headline = "Invalid request"
             
 class InvalidMetadataError(WebAPIError):
+    Headline = "Invalid metadata"
 
     def __str__(self):
-        msg = ["Invalid metadata error"]
+        msg = ["Invalid metadata"]
         for item in self.json():
             item_headline = item["message"]
             index = item.get("index")
@@ -106,20 +128,25 @@ class HTTPClient(object):
 
     def send_request(self, method, uri_suffix, headers=None, timeout=None, **args):
         self.LastURL = url = "%s/%s" % (self.ServerURL, uri_suffix)
-        if headers:
-            headers = headers.copy()
+        default_headers = {
+            "Accept": "text/plain, application/json, text/json, application/json-seq"
+        }
         if self.Token is not None:
-            headers = headers or {}
-            headers["X-Authentication-Token"] = self.Token.encode()
+            default_headers["X-Authentication-Token"] = self.Token.encode()
+        if headers:
+            default_headers.update(headers)
+        headers = default_headers
         self.LastResponse = response = self.retry_request(method, url, headers=headers, **args)
         #print(response, response.text)
         self.LastStatusCode = response.status_code
         if response.status_code == INVALID_METADATA_ERROR_CODE:
-            raise InvalidMetadataError(url, response.status_code, response.text)
+            raise InvalidMetadataError(url, response)
         if response.status_code == 404:
-            raise NotFoundError(url, response.status_code, response.text)
-        elif response.status_code != 200:
-            raise WebAPIError(url, response.status_code, response.text)
+            raise NotFoundError(url, response)
+        if response.status_code == 400:
+            raise BadRequestError(url, response)
+        elif response.status_code/100 != 2:
+            raise WebAPIError(url, response)
         return response
 
     def get_text(self, uri_suffix):
@@ -130,13 +157,11 @@ class HTTPClient(object):
 
     def unpack_json(self, json_text):
         results = json.loads(json_text)
-        if isinstance(results, dict):
-            if "results" in results:
-                results = results["results"]
-            elif "error" in results:
-                message =  "Server side error: " + results["error"]["type"] + ": "
-                message += results["error"]["value"]
-                raise ServerError(self.LastURL, self.LastStatusCode, message)
+        #if isinstance(results, dict):
+        #    if "results" in results:
+        #        results = results["results"]
+        #    elif "error" in results:
+        #        raise ServerReportedError(self.LastURL, self.LastStatusCode, results["error"]["type"], results["error"].get("value", ""))
         return results
                 
     def get_json(self, uri_suffix):
@@ -145,7 +170,34 @@ class HTTPClient(object):
     def post_json(self, uri_suffix, data):
         if not isinstance(data, (str, bytes)):
             data = json.dumps(data)
-        return self.unpack_json(self.send_request("post", uri_suffix, data=data).text)
+        headers = {"Content-Type": "text/json"}
+        return self.unpack_json(self.send_request("post", uri_suffix, data=data, headers=headers).text)
+
+    def get_json_stream(self, uri_suffix):
+        url = "%s/%s" % (self.ServerURL, uri_suffix)
+        headers = {"Accept": "application/json-seq"}
+        if self.Token is not None:
+            headers["X-Authentication-Token"] = self.Token.encode()
+
+        with self.retry_request("get", url, headers=headers, stream=True) as response:
+            if response.status_code == INVALID_METADATA_ERROR_CODE:
+                raise InvalidMetadataError(url, response.status_code, response.text)
+            if response.status_code == 404:
+                raise NotFoundError(url, response.status_code, response.text)
+            elif response.status_code != 200:
+                raise WebAPIError(url, response.status_code, response.text)
+            
+            if response.headers.get("Content-Type") != "application/json-seq":
+                raise WebAPIError(url, 200, "Expected content type application/json-seq. Got %s instead." % (response.headers.get("Content-Type"),))
+
+            for line in response.iter_lines():
+                if line:    line = line.strip()
+                while line.startswith(b'\x1E'):
+                    line = line[1:]
+                if line:
+                    #print(f"stream line:[{line}]")
+                    obj = json.loads(line)
+                    yield obj
 
 class MetaCatClient(HTTPClient, TokenAuthClientMixin):
     
@@ -195,6 +247,16 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         self.AuthURL = auth_server_url or server_url + "/auth"
         self.MaxConcurrent = max_concurrent_queries
         self.AsyncQueue = None
+        
+    def sanitize(self, *words, **kw):
+        for w in words:
+            if "'" in words:
+                raise InvalidArgument("Invalid value: %s" % (w,))
+        for name, value in kw.items():
+            if "'" in value:
+                raise InvalidArgument("Invalid value for %s: %s" % (name, value))
+            if "'" in name:
+                raise InvalidArgument("Invalid name for: %s" % (name,))
 
     @property
     def async_queue(self):
@@ -330,7 +392,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             return None
         
         
-    def create_dataset(self, spec, frozen=False, monotonic=False, metadata=None, metadata_requirements=None, 
+    def create_dataset(self, did, frozen=False, monotonic=False, metadata=None, metadata_requirements=None, 
             files_query=None, subsets_query=None,
             description=""):
 
@@ -338,7 +400,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
 
         Parameters
         ----------
-        spec : str
+        did : str
             "namespace:name"
         frozen : bool
         monotonic : bool
@@ -358,7 +420,8 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             created dataset attributes
         """
 
-        namespace, name = spec.split(":",1)     
+        namespace, name = did.split(":",1)
+        #self.sanitize(namespace=namespace, name=name)
         params = {
             "namespace":    namespace,
             "name":         name,
@@ -572,39 +635,40 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         
         Notes
         -----
+        Each file to be declared must be represented with a dictionary. The dictionary must contain one of:
         
-            Each file to be declared must be represented with a dictionary. The dictionary must contain elements:
-                one of:
-                    "did" - string in the format "<namespace>:<name>"
-                    "name" - file name and optionaly "namespace". If namespace is not present, the ``namespace`` argument will be used
-                             as the default namespace
-                    "auto_name" - pattern to auto-generate file name
-        
-            .. code-block:: python
-        
-                { 
-                    "namespace": "namespace",           # optional, namespace can be specified for each file explicitly or implicitly using the namespace=... argument
-                    "name": "filename",                 # optional,
-                    "did": "namespace:filename",        # optional, convenience for Rucio users
-                                                        # either "did" or "name", "namespace" must be present
-                    "size": ...,                        # required, integer number of bytes
-                    "metadata": {...},                  # optional, file metadata, a dictionary with arbitrary JSON'able contents
-                    "fid":  "...",                      # optional, file id. Will be auto-generated if unspecified.
-                                                        # if specified, must be unique
-                    "parents": [...],                   # optional, list of dicts, one dict per parent. See below.
-                    "checksums": {                      # optional, checksums dictionary
-                        "method": "value",...
-                    },
-                    "auto_name": "..."                  # optional, pattern to auto-generate file name if name is not specified or null
-                },...
-        
-            Parents are specified with dictionaries, one dictionary per file. Each dictionary specifies the parent file in one of three ways:
+        "did" - string in the format "<namespace>:<name>"
 
-                - "did": "<namespace>:<name>"
-                - "namespace":"...", "name":"..."
-                - "fid": "<file id>"
-        
-            DEPRECATED: if the parent is specified with a string instead of a dictionary, it is interpreferd as the parent file id.
+        "name" - file name and optionaly "namespace". If namespace is not present, the ``namespace`` argument will be used
+                 as the default namespace
+
+        "auto_name" - pattern to auto-generate file name
+    
+        .. code-block:: python
+    
+            { 
+                "namespace": "namespace",           # optional, namespace can be specified for each file explicitly or implicitly using the namespace=... argument
+                "name": "filename",                 # optional,
+                "did": "namespace:filename",        # optional, convenience for Rucio users
+                                                    # either "did" or "name", "namespace" must be present
+                "size": ...,                        # required, integer number of bytes
+                "metadata": {...},                  # optional, file metadata, a dictionary with arbitrary JSON'able contents
+                "fid":  "...",                      # optional, file id. Will be auto-generated if unspecified.
+                                                    # if specified, must be unique
+                "parents": [...],                   # optional, list of dicts, one dict per parent. See below.
+                "checksums": {                      # optional, checksums dictionary
+                    "method": "value",...
+                },
+                "auto_name": "..."                  # optional, pattern to auto-generate file name if name is not specified or null
+            },...
+    
+        Parents are specified with dictionaries, one dictionary per file. Each dictionary specifies the parent file in one of three ways:
+
+            - "did": "<namespace>:<name>"
+            - "namespace":"...", "name":"..."
+            - "fid": "<file id>"
+    
+        DEPRECATED: if the parent is specified with a string instead of a dictionary, it is interpreferd as the parent file id.
         """        
         
         default_namespace = namespace
@@ -964,7 +1028,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         return self.get_json(url)
         
     def get_namespace(self, name):
-        """Creates new namespace
+        """Get information about a snamespace
         
         Parameters
         ----------
@@ -983,7 +1047,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             return None
         
     def get_namespaces(self, names):
-        """Creates new namespace
+        """Get information for multiple namespaces
         
         Parameters
         ----------
@@ -999,7 +1063,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         return self.post_json(f"data/namespaces", names)
         
     def list_namespaces(self, pattern=None, owner_user=None, owner_role=None, directly=False):
-        """Creates new namespace
+        """List namespaces
         
         Parameters
         ----------
@@ -1034,4 +1098,37 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         for item in lst:
             if pattern is None or fnmatch.fnmatch(item["name"], pattern):
                 yield item
-    
+                
+    #
+    # Categiries
+    #
+    def list_categories(self, root=None):
+        """List namespaces
+
+        Parameters
+        ----------
+        root : str
+            Optional, if present, list only categories under the root
+
+        Returns
+        -------
+        list 
+            List of dictionaries with category information sorted by category path
+        """
+        lst = self.get_json("data/categories")
+        if root:
+            if not root.endswith('.'):
+                root += '.'
+            lst = [cat for cat in lst if cat["path"].startswith(root)]
+        return sorted(lst, key=lambda c: c["path"])
+
+    def get_category(self, path):
+        """Get category information
+        
+        Returns
+        -------
+        dict 
+            A dictionary with category information or None if not found
+        """
+        out = self.get_json(f"data/category/{path}")
+        return out
